@@ -9,27 +9,26 @@ use Exporter 5.57              qw( import );
 use Fcntl                      qw( :flock :seek );
 use File::Basename               ( );
 use File::Copy                   ( );
-use File::DataClass::Constants qw( CYGWIN EVIL EXCEPTION_CLASS FALSE
-                                   LOCK_BLOCKING LOCK_NONBLOCKING
-                                   NO_UMASK_STACK NUL PERMS STAT_FIELDS
-                                   TILDE TRUE );
+use File::DataClass::Constants qw( CYGWIN EXCEPTION_CLASS FALSE LOCK_BLOCKING
+                                   LOCK_NONBLOCKING MSOFT NO_UMASK_STACK NUL
+                                   PERMS STAT_FIELDS TILDE TRUE );
 use File::DataClass::Functions qw( ensure_class_loaded first_char is_arrayref
                                    is_coderef is_hashref is_member thread_id
                                    throw );
 use File::Spec                   ( );
-use File::Spec::Functions      qw( curdir );
+use File::Spec::Functions      qw( curdir updir );
 use IO::Dir;
 use IO::File;
 use List::Util                 qw( first );
 use Scalar::Util               qw( blessed );
 use Type::Utils                qw( enum );
-use Unexpected::Functions      qw( PathNotFound Unspecified );
+use Unexpected::Functions      qw( InvocantUndefined PathNotFound Unspecified );
 use Unexpected::Types          qw( ArrayRef Bool CodeRef Int Maybe Object
                                    PositiveInt RegexpRef SimpleStr Str );
 
 use namespace::clean -except => [ 'import', 'meta' ];
-use overload '""'       => sub { $_[ 0 ]->pathname },
-             'bool'     => sub { $_[ 0 ]->boolean  },
+use overload '""'       => sub { $_[ 0 ]->as_string  },
+             'bool'     => sub { $_[ 0 ]->as_boolean },
              'fallback' => TRUE;
 
 our @EXPORT    = qw( io );
@@ -39,7 +38,7 @@ my  $IO_LOCK   = enum 'IO_Lock' => [ FALSE, LOCK_BLOCKING, LOCK_NONBLOCKING ];
 my  $IO_MODE   = enum 'IO_Mode' => [ qw( a a+ r r+ w w+ ) ];
 my  $IO_TYPE   = enum 'IO_Type' => [ qw( dir file ) ];
 my  $LC_OSNAME = lc $OSNAME;
-my  $NTFS      = $LC_OSNAME eq EVIL || $LC_OSNAME eq CYGWIN ? TRUE : FALSE;
+my  $NTFS      = $LC_OSNAME eq MSOFT || $LC_OSNAME eq CYGWIN ? TRUE : FALSE;
 
 # Private functions
 my $_expand_tilde = sub {
@@ -67,15 +66,11 @@ my $_should_include_path = sub {
    return (not defined $_[ 0 ] or (map { $_[ 0 ]->() } ($_[ 1 ]))[ 0 ]);
 };
 
-# Private methods
+# Attribute constructors
 my $_build_dir_pattern = sub {
-   my $self = shift; my $pat = NUL;
+   my $self = shift; my $curdir = curdir; my $updir = updir;
 
-   my $curdir = curdir; my $updir = File::Spec->updir;
-
-   $curdir and $pat  = "\Q${curdir}\E"; # uncoverable condition left
-   $curdir and $updir and $pat .= '|';  # uncoverable condition left
-   $updir  and $pat .= "\Q${updir}\E";  # uncoverable condition left
+   my $pat  = "\Q${curdir}\E|\Q${updir}\E";
 
    return qr{ \A (?:$pat) \z }mx;
 };
@@ -85,7 +80,8 @@ has 'autoclose'     => is => 'lazy', isa => Bool,           default => TRUE  ;
 has 'have_lock'     => is => 'rwp',  isa => Bool,           default => FALSE ;
 has 'io_handle'     => is => 'rwp',  isa => Maybe[Object]                    ;
 has 'is_open'       => is => 'rwp',  isa => Bool,           default => FALSE ;
-has 'mode'          => is => 'rwp',  isa => $IO_MODE,       default => 'r'   ;
+has 'mode'          => is => 'rwp',  isa => $IO_MODE | PositiveInt,
+   default          => 'r'                                                   ;
 has 'name'          => is => 'rwp',  isa => SimpleStr,      default => NUL,
    coerce           => $_coerce_name,                       lazy    => TRUE  ;
 has '_perms'        => is => 'rwp',  isa => PositiveInt,    default => PERMS,
@@ -120,9 +116,8 @@ my $_clone_one_of_us = sub {
    $self->autoclose; $self->reverse; $self->sort; # Force evaluation
 
    my $clone = { %{ $self }, %{ $params // {} } };
-   my $perms = delete $clone->{_perms};
+   my $perms = delete $clone->{_perms}; $clone->{perms} //= $perms;
 
-   $clone->{perms} //= $perms;
    return $clone;
 };
 
@@ -159,6 +154,33 @@ sub BUILD {
    return;
 }
 
+my $_proxy = sub { # Methods handled by the IO::Handle object
+   my ($proxy, $chain, $mode) = @_; no strict 'refs';
+
+   my $package = caller; defined &{ "${package}::${proxy}" } and return;
+
+   return  *{ "${package}::${proxy}" } = sub {
+      my $self = shift; defined $mode and $self->assert_open( $mode );
+
+      defined $self->io_handle or throw InvocantUndefined, [ $proxy ];
+
+      my @results = $self->io_handle->$proxy( @_ ); # Mustn't copy stack args
+
+      $self->error_check; $chain and return $self;
+
+      return wantarray ? @results : $results[ 0 ];
+   };
+};
+
+$_proxy->( 'autoflush', TRUE );
+$_proxy->( 'eof'             );
+$_proxy->( 'fileno'          );
+$_proxy->( 'flush',     TRUE );
+$_proxy->( 'getc',      FALSE, 'r' );
+$_proxy->( 'sysread',   FALSE, O_RDONLY );
+$_proxy->( 'syswrite',  FALSE, O_CREAT | O_WRONLY );
+$_proxy->( 'truncate',  TRUE );
+
 # Private methods
 my $_all_file_contents = sub {
    my $self = shift; $self->is_open or $self->assert_open;
@@ -467,6 +489,16 @@ sub appendln {
    return $self->$_println( @args );
 }
 
+sub as_boolean {
+   return ($_[ 0 ]->name || $_[ 0 ]->io_handle) ? TRUE : FALSE;
+}
+
+sub as_string {
+   my $self = shift;
+
+   return $self->name || (defined $self->io_handle && NUL.$self->io_handle);
+}
+
 sub assert {
    $_[ 0 ]->_assert( TRUE ); return $_[ 0 ];
 }
@@ -503,7 +535,7 @@ sub assert_filepath {
 }
 
 sub assert_open {
-   return $_[ 0 ]->open( $_[ 1 ] || 'r', $_[ 2 ] );
+   return $_[ 0 ]->open( $_[ 1 ] // 'r', $_[ 2 ] );
 }
 
 sub atomic {
@@ -549,10 +581,6 @@ sub block_size {
    defined $_[ 1 ] and $_[ 0 ]->_block_size( $_[ 1 ] ); return $_[ 0 ];
 }
 
-sub boolean {
-   return ($_[ 0 ]->io_handle || $_[ 0 ]->name) ? TRUE : FALSE;
-}
-
 sub buffer {
    my $self = shift;
 
@@ -594,7 +622,7 @@ sub catfile {
 sub chmod {
    my ($self, $perms) = @_;
 
-   $perms ||= $self->_perms; # uncoverable condition false
+   $perms //= $self->_perms; # uncoverable condition false
    CORE::chmod $perms, $self->name;
    return $self;
 }
@@ -719,14 +747,15 @@ sub encoding {
 sub error_check {
    my $self = shift;
 
-   $self->io_handle->can( 'error' ) or return;
-   $self->io_handle->error or return;
-   $self->$_throw( 'IO error: [_1]', [ $OS_ERROR ] );
-   return;
+   $self->io_handle->can( 'error' )
+      and $self->io_handle->error
+      and $self->$_throw( 'IO error: [_1]', [ $OS_ERROR ] );
+
+   return $self;
 }
 
 sub exists {
-   return $_[ 0 ]->name ? -e $_[ 0 ]->name : FALSE;
+   return ($_[ 0 ]->name && -e $_[ 0 ]->name) ? TRUE : FALSE;
 }
 
 sub file {
@@ -847,7 +876,7 @@ sub is_readable {
 }
 
 sub is_reading {
-   my $mode = $_[ 1 ] || $_[ 0 ]->mode; return first { $_ eq $mode } qw(r r+);
+   my $mode = $_[ 1 ] || $_[ 0 ]->mode; return first { $_ eq $mode } qw( r r+ );
 }
 
 sub is_writable {
@@ -857,7 +886,7 @@ sub is_writable {
 sub is_writing {
    my $mode = $_[ 1 ] || $_[ 0 ]->mode;
 
-   return first { $_ eq $mode } qw(a a+ w w+);
+   return first { $_ eq $mode } qw( a a+ w w+ );
 }
 
 sub iterator {
@@ -947,7 +976,7 @@ sub no_follow {
 }
 
 sub open {
-   my ($self, $mode, $perms) = @_; $mode ||= $self->mode;
+   my ($self, $mode, $perms) = @_; $mode //= $self->mode;
 
    $self->is_open
       and first_char $mode eq first_char $self->mode
@@ -1003,12 +1032,12 @@ sub read {
 }
 
 sub read_dir {
-   my $self = shift; my $dir_pat = $self->_dir_pattern; my $name;
-
-   $self->type or $self->dir; $self->assert_open;
+   my $self = shift; $self->type or $self->dir; $self->assert_open;
 
    $self->is_link and $self->_no_follow and $self->close and return;
 
+   my $dir_pat = $self->_dir_pattern; my $name;
+
    if (wantarray) {
       my @names = grep { $_ !~ $dir_pat } $self->io_handle->read;
 
@@ -1060,8 +1089,7 @@ sub rmtree {
 sub seek {
    my ($self, $posn, $whence) = @_;
 
-   $self->is_open or $self->assert_open( $LC_OSNAME eq EVIL ? 'r' : 'r+' );
-
+   $self->is_open or $self->assert_open( $LC_OSNAME eq MSOFT ? 'r' : 'r+' );
    CORE::seek $self->io_handle, $posn, $whence; $self->error_check;
    return $self;
 }
@@ -1149,7 +1177,7 @@ sub tail {
 sub tell {
    my $self = shift;
 
-   $self->is_open or $self->assert_open( $LC_OSNAME eq EVIL ? 'r' : 'r+' );
+   $self->is_open or $self->assert_open( $LC_OSNAME eq MSOFT ? 'r' : 'r+' );
 
    return CORE::tell $self->io_handle;
 }
@@ -1165,8 +1193,7 @@ sub tempfile {
       ( DIR => $tempdir, TEMPLATE => (sprintf $tmplt, $PID) );
    my $t     = $self->$_constructor( $tmpfh->filename )->file;
 
-   $t->_set_io_handle( $tmpfh );
-   $t->_set_is_open( TRUE );
+   $t->_set_io_handle( $tmpfh ); $t->_set_is_open( TRUE );
    $t->_set_mode( 'w+' );
    return $t;
 }
@@ -1203,8 +1230,7 @@ sub write {
               ? $self->io_handle->write( @args )
               : $self->io_handle->write( ${ $self->buffer }, $self->length );
 
-   $self->error_check;
-   scalar @args or $self->clear;
+   $self->error_check; scalar @args or $self->clear;
    return $length;
 }
 
@@ -1396,6 +1422,19 @@ Opens the file in append mode and calls L</print> with the passed args
 
 Opens the file in append mode and calls L</println> with the passed args
 
+=head2 as_boolean
+
+   $bool = io( 'path_to_file' )->as_boolean;
+
+Returns true if the pathname has been set or the file handle is open, returns
+false otherwise. The boolean overload calls this
+
+=head2 as_string
+
+   $path_to_file = io( 'path_to_file' )->as_string;
+
+Returns the pathname of the IO object. Overload stringifies to this
+
 =head2 assert
 
    $io = io( 'path_to_file' )->assert;
@@ -1450,6 +1489,13 @@ If the value contains C<%P> it will be replaced with the process id
 
 If the value contains C<%T> it will be replaces with the thread id
 
+=head2 autoflush
+
+   $io->autoflush( $bool );
+
+Turns autoflush on or off on the file handle. Proxy method implemented by
+L<IO::Handle>
+
 =head2 backwards
 
    $io = io( 'path_to_file' )->backwards;
@@ -1480,13 +1526,6 @@ Sets binmode to the given layer
 
 Defaults to 1024. The default block size used by the L</read> method
 
-=head2 boolean
-
-   $bool = io( 'path_to_file' )->boolean;
-
-Returns true if the pathname has been set or the file handle is open, returns
-false otherwise. The boolean overload calls this
-
 =head2 buffer
 
 The internal buffer used by L</read> and L</write>
@@ -1624,6 +1663,13 @@ Deprecated in favour of L</is_empty>
 Apply the given encoding to the open file handle and store it on the
 C<_encoding> attribute
 
+=head2 eof
+
+   $bool = $io->eof;
+
+Returns true if the file handle is at end of file. Proxy method implemented by
+L<IO::Handle>
+
 =head2 error_check
 
 Tests to see if the open file handle is showing an error and if it is
@@ -1645,6 +1691,13 @@ Initialises the current object as a file
 
 Returns the filename part of pathname
 
+=head2 fileno
+
+   $fileno = $io->fileno
+
+Return the C<fileno> of the file handle. Proxy method implemented by
+L<IO::Handle>
+
 =head2 filepath
 
    $dirname = io( 'path_to_file' )->filepath;
@@ -1659,6 +1712,12 @@ Takes a subroutine reference that is used by the L</all> methods to
 filter which entries are returned. Called with C<$_> set to each
 pathname in turn. It should return true if the entry is wanted
 
+=head2 flush
+
+   $io->flush;
+
+Flush the file handle. Proxy method implemented by L<IO::Handle>
+
 =head2 getline
 
    $line = io( 'path_to_file' )->getline;
@@ -1668,6 +1727,13 @@ handle. Chomp the line if the C<_chomp> attribute is true. Check for
 errors. Close the file if the C<autoclose> attribute is true and end
 of file has been read past
 
+=head2 getc
+
+   $one_character = $io->getc;
+
+Reads one character from the file handle. Proxy method implemented by
+L<IO::Handle>
+
 =head2 getlines
 
    @lines = io( 'path_to_file' )->getlines;
@@ -1980,6 +2046,19 @@ Returns undefined if the file does not exist or the file handle is not open
 Substitutes C<$search> regular expression for C<$replace> string on each
 line of the given file
 
+=head2 sysread
+
+   $red = $io->sysread( $buffer, $length, $offset );
+
+Raw read bypasses the line buffering. Proxy method implemented by L<IO::Handle>
+
+=head2 syswrite
+
+   $wrote = $io->syswrite( $buffer, $length, $offset );
+
+Write the buffer to the file by-passing the line buffering. Proxy method
+implemented by L<IO::Handle>
+
 =head2 tail
 
    @lines = io( 'path_to_file' )->tail( $no_of_lines );
@@ -2017,6 +2096,13 @@ file system permissions which default to 0644 octal. If the file
 already exists update it's last modified datetime stamp. If a value
 for C<$time> is provided use that instead if the C<CORE::time>
 
+=head2 truncate
+
+   $io->truncate( $length );
+
+Truncate the file at the specified length.  Proxy method implemented by
+L<IO::Handle>
+
 =head2 unlink
 
    $bool = io( 'path_to_file' )->unlink;
