@@ -15,57 +15,62 @@ use Unexpected::Functions      qw( RecordNotFound Unspecified );
 
 my $class_stash = {};
 
-has 'list_class'    => is => 'ro', isa => ClassName,
+# Private functions
+my $_build_operators = sub {
+   return {
+      'eq' => sub { return $_[ 0 ] eq $_[ 1 ] },
+      '==' => sub { return $_[ 0 ] == $_[ 1 ] },
+      'ne' => sub { return $_[ 0 ] ne $_[ 1 ] },
+      '!=' => sub { return $_[ 0 ] != $_[ 1 ] },
+      '>'  => sub { return $_[ 0 ] >  $_[ 1 ] },
+      '>=' => sub { return $_[ 0 ] >= $_[ 1 ] },
+      '<'  => sub { return $_[ 0 ] <  $_[ 1 ] },
+      '<=' => sub { return $_[ 0 ] <= $_[ 1 ] },
+      '=~' => sub { my $re = $_[ 1 ]; return $_[ 0 ] =~ qr{ $re }mx },
+      '!~' => sub { my $re = $_[ 1 ]; return $_[ 0 ] !~ qr{ $re }mx },
+   };
+};
+
+# Public attributes
+has 'list_class'    => is => 'ro',   isa => ClassName,
    default          => 'File::DataClass::List';
 
-has 'result_class'  => is => 'ro', isa => ClassName,
+has 'result_class'  => is => 'ro',   isa => ClassName,
    default          => 'File::DataClass::Result';
 
-has 'result_source' => is => 'ro', isa => Object,
+has 'result_source' => is => 'ro',   isa => Object,
    handles          => [ qw( attributes defaults label_attr path storage ) ],
    required         => TRUE, weak_ref => TRUE;
 
 has '_iterator'     => is => 'rw',   isa => Int, default => 0,
    init_arg         => undef;
 
-has '_operators'    => is => 'lazy', isa => HashRef;
+has '_operators'    => is => 'lazy', isa => HashRef,
+   builder          => $_build_operators;
 
 has '_results'      => is => 'rw',   isa => ArrayRef,
-   default          => sub { [] }, init_arg => undef;
-
-# Construction
-sub _build__operators {
-   return {
-      q(eq) => sub { return $_[ 0 ] eq $_[ 1 ] },
-      q(==) => sub { return $_[ 0 ] == $_[ 1 ] },
-      q(ne) => sub { return $_[ 0 ] ne $_[ 1 ] },
-      q(!=) => sub { return $_[ 0 ] != $_[ 1 ] },
-      q(>)  => sub { return $_[ 0 ] >  $_[ 1 ] },
-      q(>=) => sub { return $_[ 0 ] >= $_[ 1 ] },
-      q(<)  => sub { return $_[ 0 ] <  $_[ 1 ] },
-      q(<=) => sub { return $_[ 0 ] <= $_[ 1 ] },
-      q(=~) => sub { my $re = $_[ 1 ]; return $_[ 0 ] =~ qr{ $re }mx },
-      q(!~) => sub { my $re = $_[ 1 ]; return $_[ 0 ] !~ qr{ $re }mx },
-   };
-}
+   builder          => sub { [] }, init_arg => undef;
 
 # Private methods
 my $_new_result_class = sub {
    my ($self, $class, $source, $values) = @_;
 
-   my $name = "${class}::".$source->name; my @attrs;
+   my $name = "${class}::".(ucfirst $source->name); my @attrs;
 
    exists $class_stash->{ $name } and return $class_stash->{ $name };
 
-   my $except = 'delete | id | insert | name | path | result_source | update';
-   my %types  = ( 'SCALAR', Maybe[Str], 'ARRAY', Maybe[ArrayRef],
-                  'HASH',   Maybe[HashRef] );
+   my $except = 'delete | id | insert | name | result_source | update';
+   my %types  = ( 'ARRAY',  Maybe[ArrayRef],
+                  'HASH',   Maybe[HashRef],
+                  'SCALAR', Maybe[Str], );
 
    for my $attr (grep { not m{ \A (?: $except ) \z }mx }
                      @{ $source->attributes }) {
-      my $type = ref $source->defaults->{ $attr } || ref $values->{ $attr };
+      my $sdef = $source->defaults->{ $attr };
+      my $type = $source->types->{ $attr }
+              || $types{ ref $sdef || ref $values->{ $attr } || 'SCALAR' };
 
-      push @attrs, $attr, [ is => 'rw', isa => $types{ $type || 'SCALAR' } ];
+      push @attrs, $attr, [ is => 'rw', isa => $type ];
    }
 
    return $class_stash->{ $name } = subclass_of
@@ -122,12 +127,11 @@ my $_splice = sub {
    my $out   = [];
 
    for my $item (@{ $items }) {
-      last unless (defined $list->[ 0 ]);
+      defined $list->[ 0 ] or last;
 
       for (0 .. $#{ $list }) {
          if ($list->[ $_ ] eq $item) {
-            CORE::splice @{ $list }, $_, 1;
-            CORE::push   @{ $out  }, $item;
+            CORE::splice @{ $list }, $_, 1; CORE::push @{ $out  }, $item;
             last;
          }
       }
@@ -281,10 +285,9 @@ sub create_or_update {
 sub delete {
    my ($self, $args) = @_; my $id = $self->$_validate_params( $args );
 
-   my $optional = (is_hashref $args) ? $args->{optional} : FALSE;
    my $path     = $self->path;
-
-   my $res = $self->$_txn_do( sub {
+   my $optional = (is_hashref $args) ? $args->{optional} : FALSE;
+   my $res      = $self->$_txn_do( sub {
       my $result; unless ($result = $self->$_find( $id )) {
          $optional or throw RecordNotFound, [ $path, $id ];
          return FALSE;
@@ -640,7 +643,7 @@ Peter Flanigan, C<< <pjfl@cpan.org> >>
 
 =head1 License and Copyright
 
-Copyright (c) 2014 Peter Flanigan. All rights reserved
+Copyright (c) 2015 Peter Flanigan. All rights reserved
 
 This program is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself. See L<perlartistic>
